@@ -1,29 +1,28 @@
 package Net::Google::Spreadsheets;
 use Moose;
-use namespace::clean -except => 'meta';
-use 5.008001;
+use 5.008;
 
 extends 'Net::Google::Spreadsheets::Base';
 
 use Carp;
 use Net::Google::AuthSub;
 use Net::Google::Spreadsheets::UserAgent;
+use Net::Google::Spreadsheets::Spreadsheet;
 
-our $VERSION = '0.06_01';
+our $VERSION = '0.01';
 
 BEGIN {
     $XML::Atom::DefaultVersion = 1;
 }
 
-has spreadsheet_feed => (
-    traits => ['Net::Google::Spreadsheets::Traits::Feed'],
+has +contents => (
     is => 'ro',
-    isa => 'Str',
-    default => 'http://spreadsheets.google.com/feeds/spreadsheets/private/full',
-    entry_class => 'Net::Google::Spreadsheets::Spreadsheet',
+    default => 'http://spreadsheets.google.com/feeds/spreadsheets/private/full'
 );
 
-sub _build_service {return $_[0]}
+has +service => (
+    default => sub {return $_[0]}
+);
 
 has source => (
     isa => 'Str',
@@ -38,35 +37,62 @@ has password => ( isa => 'Str', is => 'ro', required => 1 );
 has ua => (
     isa => 'Net::Google::Spreadsheets::UserAgent',
     is => 'ro',
-    handles => [qw(request feed entry post put)],
     required => 1,
-    lazy_build => 1,
+    lazy => 1,
+    default => sub {
+        my ($self) = @_;
+        my $authsub = Net::Google::AuthSub->new(
+            service => 'wise',
+            source => $self->source,
+        );
+        my $res = $authsub->login(
+            $self->username,
+            $self->password,
+        );
+        $res->is_success or return;
+        Net::Google::Spreadsheets::UserAgent->new(
+            source => $self->source,
+            auth => $res->auth,
+        );
+    },
+    handles => [qw(request feed entry post put)],
 );
 
-sub _build_ua {
-    my $self = shift;
-    my $authsub = Net::Google::AuthSub->new(
-        service => 'wise',
-        source => $self->source,
-    );
-    my $res = $authsub->login(
-        $self->username,
-        $self->password,
-    );
-    unless ($res && $res->is_success) {
-        croak 'Net::Google::AuthSub login failed';
-    } 
-    return Net::Google::Spreadsheets::UserAgent->new(
-        source => $self->source,
-        auth => $res->auth,
-    );
+sub spreadsheets {
+    my ($self, $args) = @_;
+    if ($args->{key}) {
+        my $atom = eval {$self->entry($self->contents.'/'.$args->{key})};
+        $atom or return;
+        return Net::Google::Spreadsheets::Spreadsheet->new(
+            atom => $atom,
+            service => $self,
+        );
+    } else {
+        my $cond = $args->{title} ? 
+        {
+            title => $args->{title},
+            'title-exact' => 'true'
+        } : {};
+        my $feed = $self->feed(
+            $self->contents,
+            $cond,
+        );
+        return grep {
+            (!%$args && 1)
+            ||
+            ($args->{title} && $_->title eq $args->{title})
+        } map {
+            Net::Google::Spreadsheets::Spreadsheet->new(
+                atom => $_, 
+                service => $self
+            )
+        } $feed->entries;
+    }
 }
 
-__PACKAGE__->meta->make_immutable;
-
-sub BUILD {
-    my ($self) = @_;
-    $self->ua; #check if login ok?
+sub spreadsheet {
+    my ($self, $args) = @_;
+    return ($self->spreadsheets($args))[0];
 }
 
 1;
@@ -81,10 +107,10 @@ Net::Google::Spreadsheets - A Perl module for using Google Spreadsheets API.
   use Net::Google::Spreadsheets;
 
   my $service = Net::Google::Spreadsheets->new(
-    username => 'mygoogleaccount@example.com',
+    username => 'myname@gmail.com', 
     password => 'mypassword'
   );
-
+  
   my @spreadsheets = $service->spreadsheets();
 
   # find a spreadsheet by key
@@ -122,7 +148,6 @@ Net::Google::Spreadsheets - A Perl module for using Google Spreadsheets API.
     {row => 1, col => 1, input_value => 'name'},
     {row => 1, col => 2, input_value => 'nick'},
     {row => 1, col => 3, input_value => 'mail'},
-    {row => 1, col => 4, input_value => 'age'},
   );
 
   # get a cell
@@ -137,17 +162,12 @@ Net::Google::Spreadsheets - A Perl module for using Google Spreadsheets API.
         name => 'Nobuo Danjou',
         nick => 'lopnor',
         mail => 'nobuo.danjou@gmail.com',
-        age  => '33',
     }
   );
 
   # fetch rows
   my @rows = $worksheet->rows;
 
-  # or fetch rows with query
-  
-  @rows = $worksheet->rows({sq => 'age > 20'});
-
   # search a row
   my $row = $worksheet->row({sq => 'name = "Nobuo Danjou"'});
 
@@ -159,43 +179,6 @@ Net::Google::Spreadsheets - A Perl module for using Google Spreadsheets API.
     }
   );
 
-  # delete the row
-  $row->delete;
-
-  # delete the worksheet
-  $worksheet->delete;
-
-  # create a table
-  my $table = $spreadsheet->add_table(
-    {
-        worksheet => $new_worksheet,
-        columns => ['name', 'nick', 'mail address', 'age'],
-    }
-  );
-
-  # add a record
-  my $record = $table->add_record(
-    {
-        name => 'Nobuo Danjou',
-        nick => 'lopnor',
-        'mail address' => 'nobuo.danjou@gmail.com',
-        age  => '33',
-    }
-  );
-
-  # find a record
-  my $found = $table->record(
-    {
-        sq => '"mail address" = "nobuo.danjou@gmail.com"'
-    }
-  );
-
-  # delete it
-  $found->delete;
-
-  # delete table
-  $table->delete;
-
 =head1 DESCRIPTION
 
 Net::Google::Spreadsheets is a Perl module for using Google Spreadsheets API.
@@ -210,7 +193,7 @@ Creates Google Spreadsheet API client. It takes arguments below:
 
 =item username
 
-Username for Google. This should be full email address format like 'mygoogleaccount@example.com'.
+Username for google. This should be full email address format like 'username@gmail.com'.
 
 =item password
 
@@ -247,53 +230,15 @@ http://spreadsheets.google.com/ccc?key=key
 
 Returns first item of spreadsheets(\%condition) if available.
 
-=head1 TESTING
-
-To test this module, you have to prepare as below.
-
-=over 4
-
-=item create a spreadsheet by hand
-
-Go to L<http://docs.google.com> and create a spreadsheet.
-
-=item set SPREADSHEET_TITLE environment variable
-
-  export SPREADSHEET_TITLE='my test spreadsheet'
-
-or so.
-
-=item set username and password for google.com via Config::Pit
-
-install Config::Pit and type 
-
-  ppit set google.com
-
-then some editor comes up and type your username and password like
-
-  ---
-  username: myname@gmail.com
-  password: foobarbaz
-
-=item run tests
-
-as always,
-
-  perl Makefile.PL
-  make
-  make test
-
-=back
-
 =head1 AUTHOR
 
 Nobuo Danjou E<lt>nobuo.danjou@gmail.comE<gt>
 
 =head1 SEE ALSO
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/developers_guide_protocol.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/developers_guide_protocol.html>
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/reference.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/reference.html>
 
 L<Net::Google::AuthSub>
 
@@ -305,10 +250,6 @@ L<Net::Google::Spreadsheets::Cell>
 
 L<Net::Google::Spreadsheets::Row>
 
-L<Net::Google::Spreadsheets::Table>
-
-L<Net::Google::Spreadsheets::Record>
-
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify
