@@ -1,15 +1,8 @@
 package Net::Google::Spreadsheets::Cell;
 use Moose;
-use namespace::clean -except => 'meta';
-use XML::Atom::Util qw(first);
 
 extends 'Net::Google::Spreadsheets::Base';
 
-has content => (
-    isa => 'Str',
-    is => 'ro',
-);
-
 has row => (
     isa => 'Int',
     is => 'ro',
@@ -26,19 +19,19 @@ has input_value => (
     trigger => sub {$_[0]->update},
 );
 
-after from_atom => sub {
+after _update_atom => sub {
     my ($self) = @_;
-    my $elem = first( $self->elem, $self->gsns->{uri}, 'cell');
+    my ($elem) = $self->elem->getElementsByTagNameNS($self->gsns->{uri}, 'cell');
     $self->{row} = $elem->getAttribute('row');
     $self->{col} = $elem->getAttribute('col');
     $self->{input_value} = $elem->getAttribute('inputValue');
     $self->{content} = $elem->textContent || '';
 };
 
-around to_atom => sub {
+around entry => sub {
     my ($next, $self) = @_;
     my $entry = $next->($self);
-    $entry->set($self->gsns, 'cell', '',
+    $entry->set($self->gsns, 'cell', '', 
         {
             row => $self->row,
             col => $self->col,
@@ -54,8 +47,6 @@ around to_atom => sub {
     return $entry;
 };
 
-__PACKAGE__->meta->make_immutable;
-
 1;
 __END__
 
@@ -68,7 +59,7 @@ Net::Google::Spreadsheets::Cell - A representation class for Google Spreadsheet
   use Net::Google::Spreadsheets;
 
   my $service = Net::Google::Spreadsheets->new(
-    username => 'mygoogleaccount@example.com',
+    username => 'myname@gmail.com',
     password => 'mypassword',
   );
 
@@ -106,9 +97,9 @@ Read only attribute. You can get the result of formula.
 
 =head1 SEE ALSO
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/developers_guide_protocol.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/developers_guide_protocol.html>
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/reference.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/reference.html>
 
 L<Net::Google::AuthSub>
 
